#include "img.h"
#include "mem_arena.h"
#include "pam.h"
#include "utils.h"
#include <stdbool.h>
#include <stddef.h>

#define SAMPLES_PER_PIXEL 4

bool init_image(Arena *arena, Image *img) {
  if (!arena || !img || img->width == 0 || img->height == 0) {
    return false;
  }

  u64 size = img->width * img->height * SAMPLES_PER_PIXEL;
  img->buf = wapp_mem_arena_alloc(arena, size);

  return img->buf != NULL;
}

void set_pixel(Image *img, u64 x, u64 y, Colour colour) {
  u64 idx = (y * img->width + x) * SAMPLES_PER_PIXEL;
  for (u64 i = 0; i < SAMPLES_PER_PIXEL; ++i) {
    img->buf[idx + i] = *(((u8 *)&colour) + i);
  }
}

void clear_image(Image *img, Colour colour) {
  for (u64 y = 0; y < img->height; ++y) {
    for (u64 x = 0; x < img->width; ++x) {
      set_pixel(img, x, y, colour);
    }
  }
}

void draw_line(Image *img, u64 x0, u64 y0, u64 x1, u64 y1, Colour colour) {
  u64 dx = absolute((i64)x1 - (i64)x0);
  u64 dy = absolute((i64)y1 - (i64)y0);
  f32 t = 0.0f;
  i64 x = 0;
  i64 y = 0;

  if (dx > dy) {
    if (x0 > x1) {
      swap(u64, x0, x1);
      swap(u64, y0, y1);
    }

    for (u64 x = x0; x < x1; ++x) {
      t = (f32)(x - x0) / (f32)(x1 - x0);
      y = (i64)y0 + ((i64)y1 - (i64)y0) * t;
      set_pixel(img, x, y, colour);
    }
  } else {
    if (y0 > y1) {
      swap(u64, x0, x1);
      swap(u64, y0, y1);
    }

    for (u64 y = y0; y < y1; ++y) {
      t = (f32)(y - y0) / (f32)(y1 - y0);
      x = (i64)x0 + ((i64)x1 - (i64)x0) * t;
      set_pixel(img, x, y, colour);
    }
  }
}

void save_image(const Image *img, const char *filename) {
  write_p7_image(img->width, img->height, img->buf, filename);
}
